@@ -21,7 +21,6 @@
 from numpy import *
 from sciproc import arraysel
 
-
 # apply a function for each element along various axis
 # data: matrix to be processed
 # axisref: a list of boolean which indicates along which axis the function has to be applied
@@ -36,6 +35,8 @@ def multifunc(data,axisref,deffunc):
     dataout = zeros(sizeout(data,axisref,deffunc)[0])
     SEL = list(repeat(None,len(shape(data))))
     SELout = list(repeat(None,len(shape(dataout))))
+
+    # todo: dataout should be able to be of arbitrary dimension and not necessarily the size of datain!
     scandata(0,axisref,SEL,data,deffunc,SELout,dataout)
     return dataout
 
@@ -55,15 +56,20 @@ def scandata(dimidx,axisref,SEL,data,deffunc,SELout,dataout):
         if (dimidx < (len(shape(data)) - 1)):
             scandata(dimidx + 1,axisref,SEL,data,deffunc,SELout,dataout)
         else:
-            dtemp= deffunc(data[arraysel(SEL,data.shape)])
+            dtempin = data[arraysel(SEL,data.shape)]
+            SHAPE = []
+            for iSHAPE in range(len(SEL)):
+                if (axisref[iSHAPE] == True):  
+                    SHAPE.append(len(SEL[iSHAPE]))
+            dtempin.shape = SHAPE
+
+            dtemp= deffunc(dtempin)
+
             if ((type(dtemp).__name__ == 'list') or (type(dtemp).__name__ == 'tuple')):
-                dataout[arraysel(SELout,dataout.shape)] = dtemp[0]
-            else:
-                dataout[arraysel(SELout,dataout.shape)] = dtemp
-
-
-
+                dtemp = dtemp[0]
 
+            # tricky!
+            dataout[arraysel(SELout,dataout.shape)] = dtemp.ravel()
 
         axidx = axidx + 1
 
@@ -97,7 +103,18 @@ def sizeout(data,axisref,deffunc):
 
     # the first item in this list contains dummy data which will not be used; 
     # the other items potentially contain the output coordinates if available
-    dummydataout = deffunc(data[arraysel(SEL,data.shape)])
+
+    # print(shape(data[arraysel(SEL,data.shape)]))
+
+    dtempin = data[arraysel(SEL,data.shape)]
+    SHAPE = []
+    for iSHAPE in range(len(SEL)):
+        if (axisref[iSHAPE] == True):  
+            SHAPE.append(len(SEL[iSHAPE]))
+    dtempin.shape = SHAPE
+
+    dummydataout = deffunc(dtempin)
+
     dimout = 0
     for dim in range(len(shape(data))):
         if (axisref[dim] == True):
